#!/usr/bin/env python
#
# Copyright 2009 Marcelo Boveto Shima.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA    02110-1301, USA.
#
# Author: marceloshima@gmail.com (Marcelo Boveto Shima)

import gconf
import os
import socket
import sys

import pyxkb
import time

from tacix.TXUtils import TXPath
from tacix.TXDBus import *

GCONF_KBD_KEY = "/desktop/gnome/peripherals/keyboard/kbd/%s"
GCONF_NL_KEY = "/desktop/gnome/peripherals/keyboard/host-%s/0/numlock_on"

class GConfKeyboard():
    def __init__(self, sessionid=None):
        self.gconfclient = gconf.client_get_default()
        if sessionid != None:
            self.txsession = get_session(sessionid)
            self.txpath = TXPath(sessionid)
            self.logger = self.txpath.get_logger("gconfkeyboard")


    def watch_resume(self):
        self.logger.info('Reapplying the keyboard settings')
        self.workaround_gnome_keyboard()

    def setup_signal(self):
        self.logger.info('Adding watcher')
        self.signal = self.txsession.connect_to_signal("SessionResumed", self.watch_resume, dbus_interface=INTERFACE_SESSION)

    def get_properties(self):
        props = {}
        for key in ('model',):
            token_value = self.gconfclient.get_entry(GCONF_KBD_KEY % key,"", False).get_value()
            if token_value is not None:
                value = token_value.get_string()
                props[key] = value

        for key in ('layouts', 'options'):
            token_value = self.gconfclient.get_entry(GCONF_KBD_KEY % key,"", False).get_value()
            nr = 0;
            if token_value is not None:
                values = token_value.get_list()
                for gconf_value in values:
                    props['keyboard_%s_%s' % (key, nr)] = gconf_value.get_string()
                    nr = nr + 1
        for key in ('num_lock',):
            hostname = socket.gethostname()
            token_value = self.gconfclient.get_entry(GCONF_NL_KEY % hostname,"", False).get_value()
            props[key] = str(token_value.get_bool())
        return props

    def set_properties(self, props, addTmpLayout=False):
        props2 = {}
        props2['layouts'] = []
        props2['options'] = []
        # Ex: {'keyboard_layouts_0': 'br', 'keyboard_layouts_1': 'us\tintl', 'model': 'abnt2', 'num_lock': 'True', 'keyboard_options_0': 'grp\tgrp:alts_toggle'}
        for key in props:
            if key in ('model',):
                gconf.client_get_default().set_string(GCONF_KBD_KEY % key, props[key])
            elif key in ('num_lock',):
                hostname = socket.gethostname()
                gconf.client_get_default().set_bool(GCONF_NL_KEY % hostname, bool(props[key]))
            else:
                keys = key.split('_')
                line = int(keys[2])
                value = props[key]
                key2 = keys[1]
                if len(props2[key2]) < line:
                    props2[key2].append(value)
                else:
                    props2[key2].insert(line, value)
        if addTmpLayout:
            props2['layouts'].append('us')
        for key in props2:
            gconf.client_get_default().set_list(GCONF_KBD_KEY % key, gconf.VALUE_STRING, props2[key])

    def workaround_gnome_keyboard(self):
        props = self.get_properties()
        self.set_properties(props, addTmpLayout=True)
        self.set_properties(props)

        pyxkb.get_active_group_index()
        if pyxkb.get_active_group_index() == 0:
            pyxkb.lock_next_group()
            time.sleep(0.1)
            pyxkb.get_active_group_index()
            pyxkb.lock_prev_group()
        else:
            pyxkb.lock_prev_group()
            time.sleep(0.1)
            pyxkb.get_active_group_index()
            pyxkb.lock_next_group()
        time.sleep(0.1)
        pyxkb.get_active_group_index()

def main(argv=None):
    gconfKeyboard = GConfKeyboard()
    gconfKeyboard.workaround_gnome_keyboard()
    return 0

if __name__ == "__main__":
    sys.exit(main())

