#!/usr/bin/python
#
# Copyright 2009 Marcelo Boveto Shima.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA    02110-1301, USA.
#
# Author: marceloshima@gmail.com (Marcelo Boveto Shima)

import glib

import pynotify

from tacix.TXUtils import TXPath
from tacix.TXDBus import *

class GTKNotification:
    def __init__ (self, sessionid):
        self.txsession = get_session(sessionid)
        self.txpath = TXPath(sessionid)
        self.logger = self.txpath.get_logger("txnotification")

        self.signal = None
        self.enable_notification = True

        if not pynotify.init("Basics"):
            self.enable_notification = False
        try:
            self.notify("TaciX Session", "Remote desktop session started.")
        except glib.GError, e:
            self.logger.debug("%s" % e)
            self.logger.debug("Disabling notification")
            self.enable_notification = False

    def watch_agent_state(self, state):
        self.logger.info("Notification watcher state is %s" % state)
        self.notify("TaciX Session", "Remote desktop session status is %s." % state)

    def watch_mount_added(self, mount_name):
        self.notify("TaciX Session", "Share %s mounted." % mount_name)

    def watch_notify(self, title, message):
        self.notify(title, message)

    def setup_signal(self):
        self.logger.debug('Adding watcher for notification')
        self.signal = self.txsession.connect_to_signal("StateChanged", self.watch_agent_state, dbus_interface=INTERFACE_SESSION)
        self.signal = self.txsession.connect_to_signal("MountAdded", self.watch_mount_added, dbus_interface=INTERFACE_SESSION)
        self.signal = self.txsession.connect_to_signal("SessionNotify", self.watch_notify, dbus_interface=INTERFACE_SESSION)
        self.logger.debug('Watcher for notification added')

    def notify(self, title, text):
        if self.enable_notification:
            n = pynotify.Notification(title, text)
            if not n.show():
                self.logger.debug("Failed to send notification")

