#!/usr/bin/env python
#
# Copyright 2009 Marcelo Boveto Shima.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA    02110-1301, USA.
#
# Author: marceloshima@gmail.com (Marcelo Boveto Shima)

def parse_nxproxy_options(options):
    opts = ['nx/nx']
    for opt_name in ("keyboard", "id", "clipboard", "geometry",
            "option", "shadow", "shadowmode", "streaming", "defer", "tile",
            "R", "render", "fullscreen", "shpix", "shmem", "encryption",
            "resize", "backingstore", "menu", "shadowuid", "clients",
            "client", "copysize", "cookie", "port", "connect",
            # Server only properties
            # "cache", "images", "type", "link"
            ):
        if opt_name in options and options[opt_name] != '':
            opts.append('%s=%s' % (opt_name, options[opt_name]))
    if 'display' not in options:
        return "%s:" % (",".join(opts))
    return "%s:%s" % (",".join(opts), options['display'])

def parse_nxproxy_cmd(options):
    return '/usr/bin/nxproxy -S %s:%s' % ('localhost', options['display'])

def parse_nxproxy_full_cmd(options):
    return 'NX_DISPLAY=%s %s' % (parse_nxproxy_nx_display_env(options), parse_nxproxy_cmd(options))

def parse_nxproxy_full_cmd_enc(options):
    return 'NX_DISPLAY=%s %s' % (parse_nxproxy_nx_display_env(options), parse_nxproxy_cmd_enc(options))

def parse_nxproxy_cmd_enc(options):
    return '/usr/bin/nxproxy -S'

def parse_nxproxy_options_file(options):
    return '%s/options' % parse_nxproxy_options_dir(options)

def parse_nxproxy_options_dir(options):
    return '$HOME/.nx/S-%s' % options['id']

def parse_nxproxy_nx_display_env(options, file=None):
    if file is not None:
        return 'nx/nx,options=%s:%s' % (file, options['display'])
    return 'nx/nx,options=%s:%s' % (parse_nxproxy_options_file(options), options['display'])

def parse_parameters_dict(options):
    ret = {}
    ret['options'] = parse_nxproxy_options(options)
    ret['options_file'] = parse_nxproxy_options_file(options)
    ret['nx_display_env'] = parse_nxproxy_nx_display_env(options)
    ret['cmd'] = parse_nxproxy_cmd(options)
    ret['full_cmd'] = parse_nxproxy_full_cmd(options)
    ret['cmd_enc'] = parse_nxproxy_cmd_enc(options)
    ret['full_cmd_enc'] = parse_nxproxy_full_cmd_enc(options)
    ret['script'] = parse_local_script(options)
    return ret

def parse_local_script(options):
    ret = "mkdir -p %s \n" % parse_nxproxy_options_dir(options)
    ret = ret + "echo \"%s\" > %s\n" % (parse_nxproxy_options(options), parse_nxproxy_options_file(options))
    ret = ret + parse_nxproxy_full_cmd(options) + "\n"
    return ret

