#!/usr/bin/env python
#
# Copyright 2009 Marcelo Boveto Shima.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA    02110-1301, USA.
#
# Author: marceloshima@gmail.com (Marcelo Boveto Shima)

import dbus
import dbus.service
import os, os.path
import subprocess
import pwd
import random

from tacix import xauth
from tacix.server.nx.options import Options
from tacix.util import *

class Shadow( ):
    def __init__( ):
        pass

    def run_dialog(self, authority_file, display):
        ret = xauth.run_program("/usr/lib/tacix/tacix-dialog", authority_file, display)
        self.logger.debug('tacix-dialog returned %d' % ret)
        self.remove_authorization()
        return ret

    def ask_cookie(self):
        self.logger.debug("Asking cookie from session %s" % self.nxoptions.ck_session_id)
        tmp_authority_file = xauth.create_xauthority_file('nx', self.nxoptions.cookie, self.nxoptions.display)
        if tmp_authority_file is None:
            return None

        if self.run_dialog(tmp_authority_file, display) == 1:
            return cookie
        return None

    def ask_console_authority(self, sender):
        display = ":" + self.nxoptions.get_property('shadowdisplay')

        ck_sessions = self.ck_manager.GetSessions()
        local_ck_session = None
        for ck_op_session in ck_sessions:
            ck_session_obj = self.bus.get_object ('org.freedesktop.ConsoleKit', ck_op_session)
            ck_session = dbus.Interface (ck_session_obj, 'org.freedesktop.ConsoleKit.Session')
            ck_display = ck_session.GetX11Display()
            self.logger.debug('Comparing %s and %s' % (display, ck_display))
            if display == ck_display:
                self.logger.debug('ConsoleKit session found')
                local_ck_session = ck_session
                break

        pwd_user = pwd.getpwuid(local_ck_session.GetUser())
        console_authority_file = '%s/.Xauthority' % pwd_user.pw_dir
        authority_file = self.nxoptions.get_property('authority_file')

        if self.bus.get_unix_user(sender) == local_ck_session.GetUnixUser():
            return xauth.merge_cookie_from_xauthority(console_authority_file, authority_file, display)

        if local_ck_session == None:
            self.logger.debug('ConsoleKit session not found')
            return False

        tmp_authority_file = "/tmp/.nx-authority-%s" % str(random.random())
        self.logger.debug('Asking to merge display %s on %s into %s' %\
                (display, console_authority_file, tmp_authority_file))
        if not xauth.merge_cookie_from_xauthority(console_authority_file, tmp_authority_file, display):
            self.logger.debug('Failed to merge from console authority')
            self.logger.debug('console = %s tmp = %s display = %s ' %\
                    (console_authority_file, tmp_authority_file, display))
            return False

        if self.run_dialog(tmp_authority_file, display) == 1:
            return xauth.merge_cookie_from_xauthority(console_authority_file, authority_file, display)
        return False


